import { CommonModule } from '@angular/common';
import * as i0 from '@angular/core';
import { Injectable, NgModule, inject } from '@angular/core';
import { Subject, mergeMap, iif, of, filter, map, from, withLatestFrom, tap, distinctUntilChanged, BehaviorSubject, observeOn, queueScheduler } from 'rxjs';
import * as i1 from 'angular-reactive-state/store/store-registry';
import { isEqual } from 'lodash';

class StateDevTools {
    constructor(storeRegistry) {
        this.storeRegistry = storeRegistry;
        this.dispatchEvents$ = new Subject();
        this.stateHistory = [];
        this.globalState = {};
    }
    /**
     * Initialize the dev tools which can be used to monitor state changes of the stores.
     */
    init() {
        if (!window.__REDUX_DEVTOOLS_EXTENSION__)
            return;
        const config = {
            features: { pause: true, export: true, test: true, jump: true },
            trace: true,
            traceLimit: 25,
        };
        this.devTools = window.__REDUX_DEVTOOLS_EXTENSION__.connect(config);
        this.devTools?.init(this.globalState);
        this.devTools?.subscribe((event) => {
            if (event.type === 'DISPATCH') {
                this.dispatchEvents$.next(event);
            }
        });
        this.watchChangesOfStores();
        this.watchChangesOfMonitor();
    }
    /**
     * Watch all changes which are coming from the monitor of the dev tools extension.
     */
    watchChangesOfMonitor() {
        this.dispatchEvents$
            .pipe(mergeMap(event => iif(() => event.type === 'DISPATCH' &&
            event.payload.type === 'JUMP_TO_ACTION', this.processDispatchEvents(event), of(event))))
            .subscribe();
    }
    /**
     * Observes all dispatch events from the devtools extension and
     * updates the respective store with the state which should be replayed.
     *
     * @param event Dispatch event from devtools extension
     * @returns
     */
    processDispatchEvents(event) {
        return of(event.state).pipe(filter(Boolean), map(state => JSON.parse(state)), mergeMap(state => from(Object.entries(state))), map(([storeName, state]) => ({
            storeName,
            state,
        })), withLatestFrom(this.storeRegistry.stores$), tap(([{ state, storeName }, store]) => {
            if (state) {
                // Temporarily save reference in history
                this.stateHistory.push(state);
                store[storeName]?.update(() => state);
            }
        }));
    }
    /**
     * Watches all state changes which are coming from all stores.
     */
    watchChangesOfStores() {
        this.storeRegistry.stores$
            .pipe(mergeMap(stores => from(Object.keys(stores)).pipe(mergeMap(key => stores[key].state$.pipe(filter(this.isStateUnprocessed.bind(this)), map(state => {
            return { name: key, state: state };
        }))))), tap(({ name, state }) => {
            this.globalState[name] = state;
            this.changeState(name, this.globalState);
        }))
            .subscribe();
    }
    /**
     * If the devtools replay some of the old states by time traveling then the references are stored inside of the devtools' state history.
     * Based on the object references the devtools recognize if the state was already processed or if it's a new state change.
     *
     * @param emittedStateFromStore
     * @returns True if state is new to the devtools and false if the state was already processed
     */
    isStateUnprocessed(emittedStateFromStore) {
        const devToolStateIndex = this.stateHistory.findIndex(historyState => Object.is(historyState, emittedStateFromStore));
        if (devToolStateIndex > -1) {
            this.stateHistory.splice(devToolStateIndex, 1);
            return false;
        }
        return true;
    }
    changeState(action, stateValue) {
        this.devTools?.send(action, stateValue);
    }
}
StateDevTools.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.0.1", ngImport: i0, type: StateDevTools, deps: [{ token: i1.StoreRegistry }], target: i0.ɵɵFactoryTarget.Injectable });
StateDevTools.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.0.1", ngImport: i0, type: StateDevTools, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.0.1", ngImport: i0, type: StateDevTools, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root',
                }]
        }], ctorParameters: function () { return [{ type: i1.StoreRegistry }]; } });

class StateDevToolsModule {
    constructor(devTools) {
        this.devTools = devTools;
        this.devTools.init();
    }
}
StateDevToolsModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.0.1", ngImport: i0, type: StateDevToolsModule, deps: [{ token: StateDevTools }], target: i0.ɵɵFactoryTarget.NgModule });
StateDevToolsModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "15.0.1", ngImport: i0, type: StateDevToolsModule, imports: [CommonModule] });
StateDevToolsModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "15.0.1", ngImport: i0, type: StateDevToolsModule, imports: [CommonModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.0.1", ngImport: i0, type: StateDevToolsModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [],
                    imports: [CommonModule],
                    providers: [],
                }]
        }], ctorParameters: function () { return [{ type: StateDevTools }]; } });

const distinctUntilObjectChanged = () => {
    return (source) => source.pipe(distinctUntilChanged(isEqual));
};

class StoreRegistry {
    constructor() {
        this.registry$ = new BehaviorSubject({});
        this.stores$ = this.registry$.asObservable();
    }
    addStore(name, storeInstance) {
        this.registry$.next({
            ...this.registry$.getValue(),
            [name]: storeInstance,
        });
    }
    deleteStore(name) {
        const stores = this.registry$.getValue();
        delete stores[name];
        this.registry$.next(stores);
    }
}
StoreRegistry.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.0.1", ngImport: i0, type: StoreRegistry, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
StoreRegistry.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.0.1", ngImport: i0, type: StoreRegistry, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.0.1", ngImport: i0, type: StoreRegistry, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root',
                }]
        }] });

class Store {
    constructor(name, initialState) {
        this.name = name;
        this.registryService = inject(StoreRegistry);
        this.stateSource = new BehaviorSubject(initialState);
        this.state$ = this.stateSource.asObservable();
        this.actionSource = new Subject();
        this.registryService.addStore(name, this);
        this.actionSource.pipe(observeOn(queueScheduler)).subscribe(action => {
            const currentState = this.stateSource.getValue();
            const nextState = action(currentState);
            this.stateSource.next(nextState);
        });
    }
    select(selector) {
        return this.state$.pipe(map(selector), map(state => structuredClone(state)), distinctUntilObjectChanged());
    }
    get snapshot() {
        return this.stateSource.getValue();
    }
    updateProperty(prop, value) {
        this.update(state => ({
            ...state,
            [prop]: value,
        }));
    }
    update(stateChanger) {
        this.actionSource.next(stateChanger);
    }
    destroy() {
        this.actionSource.complete();
        this.stateSource.complete();
        this.registryService.deleteStore(this.name);
    }
}

/*
 * Public API Surface of state
 */

/**
 * Generated bundle index. Do not edit.
 */

export { StateDevToolsModule, Store };
//# sourceMappingURL=state.mjs.map
//# sourceMappingURL=state.mjs.map
